#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2012  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Settings for administration pages
"""
import datetime

from django import forms
from django.conf import settings
from django.contrib import admin, messages
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import Q
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext_lazy as _
try:
    from chimere import tasks
except ImportError:
    pass

from chimere.forms import MarkerAdminForm, RouteAdminForm, AreaAdminForm,\
                          NewsAdminForm, CategoryAdminForm, ImporterAdminForm, \
                          PictureFileAdminForm, MultimediaFileAdminForm, OSMForm
from chimere.models import Category, Icon, SubCategory, Marker, \
     PropertyModel, News, Route, Area, ColorTheme, Color, \
     MultimediaFile, PictureFile, Importer, Layer, AreaLayers,\
     get_areas_for_user, get_users_by_area
from chimere.utils import unicode_normalize, ShapefileManager, KMLManager,\
                          CSVManager
from chimere.widgets import TextareaWidget

def disable(modeladmin, request, queryset):
    for item in queryset:
        item.status = 'D'
        item.save()
disable.short_description = _(u"Disable")

def validate(modeladmin, request, queryset):
    for item in queryset:
        item.status = 'A'
        item.save()
validate.short_description = _(u"Validate")

def export_to_kml(modeladmin, request, queryset):
    u"""
    Export data to KML
    """
    filename, result = KMLManager.export(queryset)
    response = HttpResponse(result,
                        mimetype='application/vnd.google-earth.kml+xml')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return response
export_to_kml.short_description = _(u"Export to KML")

def export_to_shapefile(modeladmin, request, queryset):
    u"""
    Export data to Shapefile
    """
    filename, zip_stream = ShapefileManager.export(queryset)
    # Stick it all in a django HttpResponse
    response = HttpResponse()
    response['Content-Disposition'] = 'attachment; filename=%s.zip' % filename
    response['Content-length'] = str(len(zip_stream))
    response['Content-Type'] = 'application/zip'
    response.write(zip_stream)
    return response
export_to_shapefile.short_description = _(u"Export to Shapefile")

def export_to_csv(modeladmin, request, queryset):
    u"""
    Export data to CSV
    """
    filename, result = CSVManager.export(queryset)
    response = HttpResponse(result, mimetype='text/csv')
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return response
export_to_csv.short_description = _(u"Export to CSV")

def managed_modified(modeladmin, request, queryset):
    # not very clean... There is must be a better way to do that
    redirect_url = request.get_full_path().split('admin_modification')[0]
    if queryset.count() != 1 and len(set([i.ref_item or i
                                         for i in queryset.all()])) != 1:
        messages.error(request, _(u"Only one item can be managed at a "
                                  u"time."))
        return HttpResponseRedirect(redirect_url)

    item = queryset.all()[0]
    if not item.ref_item or item.ref_item == item:
        try:
            item = modeladmin.model.objects.filter(ref_item=item
                                          ).exclude(pk=item.pk).all()[0]
        except IndexError:
            messages.error(request, _(u"No modified item associated "
                                      u"to the selected item."))
            return HttpResponseRedirect(redirect_url)
    item_ref = item.ref_item
    if request.POST.get('rapprochement'):
        couple = [(item, item_ref)]
        if hasattr(item, 'associated_marker'):
            couple.append((item.associated_marker.all()[0],
                           item_ref.associated_marker.all()[0]))
        updated = dict(request.POST)
        # clean
        for k in ('action', 'rapprochement', 'index', '_selected_action'):
            if k in updated:
                updated.pop(k)
        for idx, cpl in enumerate(couple):
            it, it_ref = cpl
            # don't copy geometry of associated items
            if idx:
                for k in ('route', 'point'):
                    if k in updated:
                        updated.pop(k)
            updated_keys = updated.keys()
            if it.status == 'I':
                updated_keys.append('import_version')
            for k in updated_keys:
                if k != 'import_version' and not request.POST[k]:
                    continue
                if hasattr(it_ref, k):
                    c_value = getattr(it_ref, k)
                    if hasattr(c_value, 'select_related'):
                        c_value.clear()
                        for val in getattr(it, k).all():
                            c_value.add(val)
                    else:
                        setattr(it_ref, k, getattr(it, k))
                        it_ref.save()
                elif k.startswith('property_'):
                    try:
                        pm = PropertyModel.get(pk=int(k[len('property_'):]))
                        it_ref.setProperty(pm, it.getProperty(pm))
                    except (ValueError, ObjectDoesNotExist):
                        pass
        if hasattr(item, 'associated_marker'):
            for it in item.associated_marker.all():
                it.delete()
        item.delete()
        messages.success(request, _(u"Modified item traited."))
        return HttpResponseRedirect(redirect_url)
    return render_to_response('admin/chimere/managed_modified.html',
                              {'item':item, 'item_ref':item_ref},
                              context_instance=RequestContext(request))
managed_modified.short_description = _(u"Managed modified items")

class PictureInline(admin.TabularInline):
    model = PictureFile
    extra = 1
    ordering = ('order',)
    form = PictureFileAdminForm
    readonly_fields = ('height', 'width')
    exclude = ('thumbnailfile', 'thumbnailfile_height', 'thumbnailfile_width')

class MultimediaInline(admin.TabularInline):
    model = MultimediaFile
    extra = 1
    ordering = ('order',)
    form = MultimediaFileAdminForm

class MarkerAdmin(admin.ModelAdmin):
    """
    Specialized the Point field.
    """
    search_fields = ("name",)
    list_display = ('name', 'status')
    list_filter = ('status', 'categories')
    actions = [validate, disable, managed_modified, export_to_kml,
               export_to_shapefile, export_to_csv]
    exclude = ['submiter_session_key', 'import_key', 'import_version',
               'available_date']
    readonly_fields = ['submiter_email', 'submiter_comment', 'import_source',
                       'ref_item', 'modified_since_import', 'route']
    form = MarkerAdminForm
    fieldsets = ((None, {
                    'fields': ['point', 'name', 'status', 'categories',
                               'description', 'start_date', 'end_date']
                  }),
                 (_(u"Submitter"), {
                    'classes':('collapse',),
                    'fields': ('submiter_name', 'submiter_email',
                               'submiter_comment')
                 }),
                 (_(u"Import"), {
                    'classes':('collapse',),
                    'fields': ('not_for_osm', 'modified_since_import',
                               'import_source', 'origin', 'license')
                 }),
                 (_(u"Associated items"), {
                    'classes':('collapse',),
                    'fields': ('ref_item', 'route',)
                 }),
                )
    inlines = [MultimediaInline, PictureInline]
    has_properties = True

    def __init__(self, *args, **kwargs):
        """
        Manage properties in fieldsets.
        """
        if self.has_properties:
            main_fields = self.fieldsets[0][1]['fields']
            for pm in PropertyModel.objects.filter(available=True).order_by('order'
                                               ).all():
                pm_name = pm.getNamedId()
                if pm_name not in main_fields:
                    main_fields.append(pm_name)
        super(MarkerAdmin, self).__init__(*args, **kwargs)

    def queryset(self, request):
        qs = self.model._default_manager.get_query_set()
        if not request.user.is_superuser:
            areas = get_areas_for_user(request.user)
            contained = Q()
            for area in areas:
                contained = contained | area.getIncludeMarker()
            qs = qs.filter(contained)
        ordering = self.ordering or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs.distinct()

    def admin_modification(self, request, item_id):
        '''
        Redirect to the marker modification form
        '''
        return managed_modified(self, request,
                            Marker.objects.filter(pk=item_id))
    def get_urls(self):
        from django.conf.urls.defaults import *
        urls = super(MarkerAdmin, self).get_urls()
        my_urls = patterns('',
            url(r'^admin_modification/(?P<item_id>\d+)/$',
                self.admin_site.admin_view(self.admin_modification),
                name='admin-modification'),
        )
        return my_urls + urls


class RouteAdmin(MarkerAdmin):
    """
    Specialized the Route field.
    """
    search_fields = ("name",)
    list_display = ('name', 'status')
    list_filter = ('status', 'categories')
    exclude = ['height', 'width']
    form = RouteAdminForm
    readonly_fields = ('associated_file', 'ref_item', 'has_associated_marker')
    actions = [validate, disable, managed_modified, export_to_kml,
               export_to_shapefile, export_to_csv]
    fieldsets = ((None, {
                    'fields': ['route', 'name', 'status', 'categories',
                               'start_date', 'end_date']
                  }),
                 (_(u"Submitter"), {
                    'classes':('collapse',),
                    'fields': ('submiter_name', 'submiter_email',
                               'submiter_comment')
                 }),
                 (_(u"Import"), {
                    'classes':('collapse',),
                    'fields': ('modified_since_import', 'import_source',
                               'origin', 'license')
                 }),
                 (_(u"Associated items"), {
                    'classes':('collapse',),
                    'fields': ('ref_item', 'associated_file',
                               'has_associated_marker')
                 }),
                )
    inlines = []
    has_properties = False

    def queryset(self, request):
        qs = self.model._default_manager.get_query_set()
        if not request.user.is_superuser:
            areas = get_areas_for_user(request.user)
            contained = Q()
            for area in areas:
                contained = contained | area.getIncludeRoute()
            qs = qs.filter(contained)
        ordering = self.ordering or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs

    def admin_modification(self, request, item_id):
        '''
        Redirect to the route modification form
        '''
        return managed_modified(self, request,
                                Route.objects.filter(pk=item_id))

class LayerInline(admin.TabularInline):
    model = AreaLayers
    extra = 1

class AreaAdmin(admin.ModelAdmin):
    """
    Specialized the area field.
    """
    form = AreaAdminForm
    exclude = ['upper_left_corner', 'lower_right_corner']
    inlines = [LayerInline]
    list_display = ['name', 'order', 'available', 'default']

def importing(modeladmin, request, queryset):
    for importer in queryset:
        importer.state = unicode(tasks.IMPORT_MESSAGES['import_pending'][0])
        importer.save()
        tasks.importing(importer.pk)
importing.short_description = _(u"Import")

def cancel_import(modeladmin, request, queryset):
    for importer in queryset:
        importer.state = tasks.IMPORT_MESSAGES['import_cancel'][0]
        importer.save()
cancel_import.short_description = _(u"Cancel import")

def cancel_export(modeladmin, request, queryset):
    for importer in queryset:
        importer.state = tasks.IMPORT_MESSAGES['export_cancel'][0]
        importer.save()
cancel_export.short_description = _(u"Cancel export")

def export_to_osm(modeladmin, request, queryset):
    if queryset.count() > 1:
        messages.error(request, _(u"Can manage only one OSM export at a time."))
        return HttpResponseRedirect(request.get_full_path())
    importer = queryset.all()[0]
    if Marker.objects.filter(categories__in=importer.categories.all(),
                             status='I').count():
        messages.error(request, _(u"You must treat all item with the status "\
                                  u"\"imported\" before exporting to OSM."))
        return HttpResponseRedirect(request.get_full_path())
    if importer.importer_type != 'OSM':
        messages.error(request, _(u"Only OSM importer are managed for export."))
        return HttpResponseRedirect(request.get_full_path())
    item_nb = Marker.objects.filter(status='A',
                categories=importer.categories.all(),
                not_for_osm=False, modified_since_import=True,
                route=None).count()
    if not item_nb:
        messages.error(request, _(u"No point of interest are concerned by this "
                                  u"export."))
        return HttpResponseRedirect(request.get_full_path())
    form = None
    if request.method == 'POST' and (
      'email' in request.POST or 'api' in request.POST
      or 'password' in request.POST):
        form = OSMForm(request.POST)
        if form.is_valid():
            importer.state = unicode(tasks.IMPORT_MESSAGES['export_pending'][0])
            importer.save()
            tasks.exporting(importer.pk, form.cleaned_data)
            messages.success(request, _(u"Export launched."))
            return HttpResponseRedirect(request.get_full_path())
    else:
        form = OSMForm()
    msg_item = _(u"%s point(s) of interest concerned by this export before "\
                 u"bounding box filter.") % item_nb
    return render_to_response('admin/chimere/osm_export.html', {'item':importer,
                                             'form':form, 'msg_item':msg_item},
                              context_instance=RequestContext(request))
export_to_osm.short_description = _(u"Export to osm")

class ImporterAdmin(admin.ModelAdmin):
    form = ImporterAdminForm
    list_display = ('display_categories', 'default_name', 'importer_type',
                    'source', 'state', 'filtr')
    list_filter = ('importer_type',)
    readonly_fields = ('state',)
    actions = [importing, cancel_import, export_to_osm, cancel_export]
admin.site.register(Importer, ImporterAdmin)

class PropertyModelAdmin(admin.ModelAdmin):
    list_display = ('name', 'order', 'available')

class NewsAdmin(admin.ModelAdmin):
    """
    Use the TinyMCE widget for the news content
    """
    form = NewsAdminForm

class SubcatInline(admin.TabularInline):
    model = SubCategory
    extra = 1

class CategoryAdmin(admin.ModelAdmin):
    """
    Use the TinyMCE widget for categories
    """
    form = CategoryAdminForm
    inlines = [SubcatInline]
    list_display = ['name', 'order']

class ColorInline(admin.TabularInline):
    model = Color

class ColorThemeAdmin(admin.ModelAdmin):
    inlines = [ColorInline,]

class IconAdmin(admin.ModelAdmin):
    exclude = ['height', 'width']

# register of differents database fields
admin.site.register(News, NewsAdmin)
admin.site.register(Category, CategoryAdmin)
admin.site.register(Icon, IconAdmin)
admin.site.register(Marker, MarkerAdmin)
admin.site.register(Route, RouteAdmin)
if not settings.CHIMERE_HIDE_PROPERTYMODEL:
    admin.site.register(PropertyModel, PropertyModelAdmin)
admin.site.register(Area, AreaAdmin)
admin.site.register(ColorTheme, ColorThemeAdmin)
admin.site.register(Layer)



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):
    
    def forwards(self, orm):
        
        # Adding model 'News'
        db.create_table('main_news', (
            ('available', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('date', self.gf('django.db.models.fields.DateField')(auto_now_add=True, blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')()),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['News'])

        # Adding model 'TinyUrl'
        db.create_table('main_tinyurl', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('parameters', self.gf('django.db.models.fields.CharField')(max_length=500)),
        ))
        db.send_create_signal('chimere', ['TinyUrl'])

        # Adding model 'ColorTheme'
        db.create_table('main_colortheme', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['ColorTheme'])

        # Adding model 'Color'
        db.create_table('main_color', (
            ('code', self.gf('django.db.models.fields.CharField')(max_length=6)),
            ('color_theme', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.ColorTheme'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('chimere', ['Color'])

        # Adding model 'Category'
        db.create_table('main_category', (
            ('available', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('description', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['Category'])

        # Adding model 'Icon'
        db.create_table('main_icon', (
            ('width', self.gf('django.db.models.fields.IntegerField')()),
            ('image', self.gf('django.db.models.fields.files.ImageField')(max_length=100)),
            ('height', self.gf('django.db.models.fields.IntegerField')()),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['Icon'])

        # Adding model 'SubCategory'
        db.create_table('main_subcategory', (
            ('category', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.Category'])),
            ('available', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
            ('item_type', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('color_theme', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.ColorTheme'], null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('icon', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.Icon'])),
        ))
        db.send_create_signal('chimere', ['SubCategory'])

        # Adding M2M table for field areas on 'SubCategory'
        db.create_table('main_subcategory_areas', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('subcategory', models.ForeignKey(orm['chimere.subcategory'], null=False)),
            ('area', models.ForeignKey(orm['chimere.area'], null=False))
        ))
        db.create_unique('main_subcategory_areas', ['subcategory_id', 'area_id'])

        # Adding model 'Marker'
        db.create_table('main_marker', (
            ('status', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('picture', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('end_date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('point', self.gf('chimere.widgets.PointField')()),
            ('route', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.Route'], null=True, blank=True)),
            ('height', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('width', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('start_date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('available_date', self.gf('django.db.models.fields.DateTimeField')(null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('chimere', ['Marker'])

        # Adding M2M table for field categories on 'Marker'
        db.create_table('main_marker_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('marker', models.ForeignKey(orm['chimere.marker'], null=False)),
            ('subcategory', models.ForeignKey(orm['chimere.subcategory'], null=False))
        ))
        db.create_unique('main_marker_categories', ['marker_id', 'subcategory_id'])

        # Adding model 'RouteFile'
        db.create_table('main_routefile', (
            ('file_type', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('raw_file', self.gf('django.db.models.fields.files.FileField')(max_length=100)),
            ('simplified_file', self.gf('django.db.models.fields.files.FileField')(max_length=100, null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['RouteFile'])

        # Adding model 'Route'
        db.create_table('main_route', (
            ('status', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('picture', self.gf('django.db.models.fields.files.ImageField')(max_length=100, null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('end_date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('route', self.gf('chimere.widgets.RouteField')()),
            ('height', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('width', self.gf('django.db.models.fields.IntegerField')(null=True, blank=True)),
            ('associated_file', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.RouteFile'], null=True, blank=True)),
            ('start_date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal('chimere', ['Route'])

        # Adding M2M table for field categories on 'Route'
        db.create_table('main_route_categories', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('route', models.ForeignKey(orm['chimere.route'], null=False)),
            ('subcategory', models.ForeignKey(orm['chimere.subcategory'], null=False))
        ))
        db.create_unique('main_route_categories', ['route_id', 'subcategory_id'])

        # Adding model 'Area'
        db.create_table('main_area', (
            ('available', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
            ('urn', self.gf('django.db.models.fields.SlugField')(db_index=True, unique=True, max_length=50, blank=True)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('upper_left_corner', self.gf('django.contrib.gis.db.models.fields.PointField')(default='POINT(0 0)')),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
            ('lower_right_corner', self.gf('django.contrib.gis.db.models.fields.PointField')(default='POINT(0 0)')),
        ))
        db.send_create_signal('chimere', ['Area'])

        # Adding model 'PropertyModel'
        db.create_table('main_propertymodel', (
            ('available', self.gf('django.db.models.fields.BooleanField')(default=False, blank=True)),
            ('order', self.gf('django.db.models.fields.IntegerField')()),
            ('type', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=150)),
        ))
        db.send_create_signal('chimere', ['PropertyModel'])

        # Adding model 'Property'
        db.create_table('main_property', (
            ('marker', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.Marker'])),
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('value', self.gf('django.db.models.fields.TextField')()),
            ('propertymodel', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['chimere.PropertyModel'])),
        ))
        db.send_create_signal('chimere', ['Property'])
    
    
    def backwards(self, orm):
        
        # Deleting model 'News'
        db.delete_table('main_news')

        # Deleting model 'TinyUrl'
        db.delete_table('main_tinyurl')

        # Deleting model 'ColorTheme'
        db.delete_table('main_colortheme')

        # Deleting model 'Color'
        db.delete_table('main_color')

        # Deleting model 'Category'
        db.delete_table('main_category')

        # Deleting model 'Icon'
        db.delete_table('main_icon')

        # Deleting model 'SubCategory'
        db.delete_table('main_subcategory')

        # Removing M2M table for field areas on 'SubCategory'
        db.delete_table('main_subcategory_areas')

        # Deleting model 'Marker'
        db.delete_table('main_marker')

        # Removing M2M table for field categories on 'Marker'
        db.delete_table('main_marker_categories')

        # Deleting model 'RouteFile'
        db.delete_table('main_routefile')

        # Deleting model 'Route'
        db.delete_table('main_route')

        # Removing M2M table for field categories on 'Route'
        db.delete_table('main_route_categories')

        # Deleting model 'Area'
        db.delete_table('main_area')

        # Deleting model 'PropertyModel'
        db.delete_table('main_propertymodel')

        # Deleting model 'Property'
        db.delete_table('main_property')
    
    
    models = {
        'chimere.area': {
            'Meta': {'object_name': 'Area', 'db_table': "'main_area'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'lower_right_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'upper_left_corner': ('django.contrib.gis.db.models.fields.PointField', [], {'default': "'POINT(0 0)'"}),
            'urn': ('django.db.models.fields.SlugField', [], {'db_index': 'True', 'unique': 'True', 'max_length': '50', 'blank': 'True'})
        },
        'chimere.category': {
            'Meta': {'object_name': 'Category', 'db_table': "'main_category'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.color': {
            'Meta': {'object_name': 'Color', 'db_table': "'main_color'"},
            'code': ('django.db.models.fields.CharField', [], {'max_length': '6'}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.colortheme': {
            'Meta': {'object_name': 'ColorTheme', 'db_table': "'main_colortheme'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.icon': {
            'Meta': {'object_name': 'Icon', 'db_table': "'main_icon'"},
            'height': ('django.db.models.fields.IntegerField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'width': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.marker': {
            'Meta': {'object_name': 'Marker', 'db_table': "'main_marker'"},
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'point': ('chimere.widgets.PointField', [], {}),
            'route': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Route']", 'null': 'True', 'blank': 'True'}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.news': {
            'Meta': {'object_name': 'News', 'db_table': "'main_news'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {}),
            'date': ('django.db.models.fields.DateField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '150'})
        },
        'chimere.property': {
            'Meta': {'object_name': 'Property', 'db_table': "'main_property'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'marker': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Marker']"}),
            'propertymodel': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.PropertyModel']"}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        'chimere.propertymodel': {
            'Meta': {'object_name': 'PropertyModel', 'db_table': "'main_propertymodel'"},
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {}),
            'type': ('django.db.models.fields.CharField', [], {'max_length': '1'})
        },
        'chimere.route': {
            'Meta': {'object_name': 'Route', 'db_table': "'main_route'"},
            'associated_file': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.RouteFile']", 'null': 'True', 'blank': 'True'}),
            'categories': ('chimere.widgets.SelectMultipleField', [], {'to': "orm['chimere.SubCategory']", 'symmetrical': 'False'}),
            'end_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'height': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'picture': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'route': ('chimere.widgets.RouteField', [], {}),
            'start_date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'width': ('django.db.models.fields.IntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        'chimere.routefile': {
            'Meta': {'object_name': 'RouteFile', 'db_table': "'main_routefile'"},
            'file_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'raw_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100'}),
            'simplified_file': ('django.db.models.fields.files.FileField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'})
        },
        'chimere.subcategory': {
            'Meta': {'object_name': 'SubCategory', 'db_table': "'main_subcategory'"},
            'areas': ('chimere.widgets.SelectMultipleField', [], {'symmetrical': 'False', 'related_name': "'areas'", 'blank': 'True', 'to': "orm['chimere.Area']"}),
            'available': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'category': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Category']"}),
            'color_theme': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.ColorTheme']", 'null': 'True', 'blank': 'True'}),
            'icon': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['chimere.Icon']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'item_type': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '150'}),
            'order': ('django.db.models.fields.IntegerField', [], {})
        },
        'chimere.tinyurl': {
            'Meta': {'object_name': 'TinyUrl', 'db_table': "'main_tinyurl'"},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'parameters': ('django.db.models.fields.CharField', [], {'max_length': '500'})
        }
    }
    
    complete_apps = ['chimere']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2012  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

from django.conf import settings
if 'kombu.transport.django' in settings.INSTALLED_APPS \
   and 'djcelery' in settings.INSTALLED_APPS:
    from celery.decorators import task

from django.core.cache import cache
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _

from chimere.models import Importer
from external_utils import OsmApi


def single_instance_task(timeout):
    def task_exc(func):
        def wrapper(*args, **kwargs):
            return func()
        return wrapper
    return task_exc

if 'task' in globals():
    def single_instance_task(timeout):
        def task_exc(func):
            def wrapper(*args, **kwargs):
                lock_id = "celery-single-instance-" + func.__name__
                acquire_lock = lambda: cache.add(lock_id, "true", timeout)
                release_lock = lambda: cache.delete(lock_id)
                if acquire_lock():
                    try:
                        func(*args, **kwargs)
                    finally:
                        release_lock()
            return wrapper
        return task_exc
else:
    def task():
        def task_exc(func):
            def wrapper(*args, **kwargs):
                func(*args, **kwargs)
            return wrapper
        return task_exc

IMPORT_MESSAGES = {
                   'import_pending':[_(u"Import pending")],
                   'import_process':[_(u"Import processing")],
                   'import_done':[_(u"Import successfuly done"),
                    _(u" %(new)d new item(s), %(updated)d updated item(s)")],
                   'import_failed':[_(u"Import failed"), "%s"],
                   'import_cancel':[_(u"Import canceled")],
                   'export_pending':[_(u"Export pending")],
                   'export_process':[_(u"Export processing")],
                   'export_done':[_(u"Export successfuly done"),
                    _(u" %(updated)d updated item(s)")],
                   'export_failed':[_(u"Export failed"), "%s"],
                   'export_cancel':[_(u"Export canceled")]
                   }

@task()
def importing(importer_pk):
    try:
        importer = Importer.objects.get(pk=importer_pk)
    except ObjectDoesNotExist:
        # importer doesn't exists anymore: too late!
        return
    if importer.state != IMPORT_MESSAGES['import_pending'][0]:
        # import canceled or done
        return
    importer.state = unicode(IMPORT_MESSAGES['import_process'][0])
    importer.save()
    new_item, updated_item, error = importer.manager.get()
    importer.state = error + ' ' if error else ''
    importer.state += unicode(IMPORT_MESSAGES['import_done'][0])
    importer.state += u" - " \
         + unicode(IMPORT_MESSAGES['import_done'][1]) % {'new':new_item,
                                                         'updated':updated_item}
    importer.state = importer.state[:200]
    importer.save()
    return True

@task()
@single_instance_task(60*10)
def exporting(importer_pk, extra_args=[]):
    try:
        importer = Importer.objects.get(pk=importer_pk)
    except ObjectDoesNotExist:
        # importer doesn't exists anymore: too late!
        return
    if importer.state != IMPORT_MESSAGES['export_pending'][0]:
        # import canceled or done
        return
    importer.state = unicode(IMPORT_MESSAGES['export_process'][0])
    importer.save()
    error = None
    try:
        updated_item, error = importer.manager.put(extra_args)
    except OsmApi.ApiError, error:
        pass
    if error:
        importer.state = unicode(IMPORT_MESSAGES['export_failed'][0]) \
              + u" - " + unicode(IMPORT_MESSAGES['export_failed'][1]) % error
        importer.save()
        return
    importer.state = unicode(IMPORT_MESSAGES['export_done'][0]) + u" - " \
         + unicode(IMPORT_MESSAGES['export_done'][1]) % {'updated':updated_item}
    importer.save()
    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2008-2010  Étienne Loks  <etienne.loks_AT_peacefrogsDOTnet>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# See the file COPYING for details.

"""
Actions available in the main interface
"""
from django.conf import settings
from django.contrib.auth import models
from django.utils.translation import ugettext_lazy as _

class Action:
    def __init__(self, id, path, label):
        self.id, self.path, self.label = id, path, label

actions = [(Action('view', '', _('View')), []),
           (Action('contribute', 'edit/', _('Contribute')),
                    (Action('edit', 'edit/', _('Add a new point of interest')),
                     Action('edit-route', 'edit-route/', _('Add a new route'))),
           ),]

if settings.CHIMERE_FEEDS:
    actions.append((Action('rss', 'feeds', _('RSS feeds')), []))

if settings.EMAIL_HOST:
    actions.append((Action('contact', 'contact', _('Contact us')), []),)

